var $ = document.querySelector.bind(document);

var canvas = $("#field"),
	ctx = canvas.getContext("2d"),
	frm = $("#frm");

var xRenderScale = canvas.width / 200,
	yRenderScale = canvas.height / 200;

function httpGet(url, callback) {
	var xhr = new XMLHttpRequest();
	xhr.onload = function() {
		callback(xhr.responseText);
	};
	xhr.open("GET", url);
	xhr.send();
}

var running = false, startTime;

var xOffset = 0, yOffset = 0, scale = 1;

frm.onsubmit = function(e) {
	e.preventDefault();

	state = null;
	engines = { up: false, left: false, right: false };
	server = frm.addr.value + ":" + parseInt(frm.port.value, 10);

	init();
};

var lunarModule = new Image();
lunarModule.src = "lunar_lander.png";

// render every frame
function render() {
	if(!running) {
		requestAnimationFrame(render);
		return;
	}

	ctx.fillStyle = "#fff";
	ctx.fillRect(0, 0, canvas.width, canvas.height);

	if(!lastResp) {
		requestAnimationFrame(render);
		return;
	}

	var state = getState(lastResp, engines, (Date.now() - lastResp.timestamp) / 1000);

	state.y = Math.max(state.y, 10);

	var modulePosition = {
		x: (state.x - 10 - xOffset) * scale,
		y: (state.y - 10 - yOffset) * scale
	};

	var linePosition = {
		x1: (lastResp.l - xOffset) * scale,
		x2: (lastResp.r - xOffset) * scale
	};

	var minX = Math.min(modulePosition.x, linePosition.x1),
		maxX = Math.max(modulePosition.x, linePosition.x2);

	var xRange = Math.max(200, maxX - minX);

	var minY = 0,
		maxY = modulePosition.y;

	var yRange = maxY - minY;

	// xOffset = (lastResp.l + lastResp.r) / 2 - 100; // vycentrovat na plochu
	// scale = Math.min(200 / xRange, 200 / yRange);

	// modulePosition = {
	// 	x: (state.x - 10 - xOffset) * scale,
	// 	y: (state.y - 10 - yOffset) * scale
	// };

	// linePosition = {
	// 	x1: (lastResp.l - xOffset) * scale,
	// 	x2: (lastResp.r - xOffset) * scale
	// };

	// draw module
	ctx.drawImage(
		lunarModule,
		modulePosition.x * xRenderScale,
		(200 / scale - modulePosition.y) * yRenderScale - 20 * yRenderScale * scale,
		20 * xRenderScale * scale,
		20 * yRenderScale * scale
	);

	// draw line
	ctx.beginPath();
	ctx.moveTo(linePosition.x1 * xRenderScale, 200 * yRenderScale);
	ctx.lineTo(linePosition.x2 * xRenderScale, 200 * yRenderScale);
	ctx.lineWidth = 2 * yRenderScale * scale;
	ctx.strokeStyle = "#000";
	ctx.stroke();

	$("#spdX").textContent = state.vx;
	$("#spdBarX").style.width = Math.min(state.vx + 15, 30) * 10 + "px";
	$("#spdBarX").style.backgroundColor = Math.abs(state.vx) <= 5 ? "#0f0" : "#f00";

	$("#spdY").textContent = state.vy;
	$("#spdBarY").style.width = Math.min(state.vy + 15, 30) * 10 + "px";
	$("#spdBarY").style.backgroundColor = Math.abs(state.vy) <= 5 ? "#0f0" : "#f00";

	$("#fuel").textContent = state.p;
	$("#fuelBar").style.width = Math.min(state.p, 30) * 10 + "px";
	$("#fuelBar").style.backgroundColor = state.p >= 5 ? "#0f0" : "#f00";

	$("#engLeft").style.backgroundColor = engines.left ? "#0f0" : "#f00";
	$("#engUp").style.backgroundColor = engines.up ? "#0f0" : "#f00";
	$("#engRight").style.backgroundColor = engines.right ? "#0f0" : "#f00";

	requestAnimationFrame(render);
}
requestAnimationFrame(render);

// update state
function updateState() {
	if(!running)
		return;

	step(lastResp.id, engines, function(msg) {
		if(!running)
			return;

		if(!msg)
			return;

		// game finished
		running = false;
		var time = (Date.now() - startTime) / 1000;

		alert(msg + " (" + time + " s)");
	});
}
setInterval(updateState, 500);

function init() {
	console.log("Starting...");

	start(function() {
		running = true;
		startTime = Date.now();
	});
}

window.onkeydown = function(e) {
	if([ 37, 38, 39 ].indexOf(e.keyCode) === -1 || !running)
		return;

	e.preventDefault();

	switch(e.keyCode) {
		case 37:
			engines.left = true;
			break;
		case 38:
			engines.up = true;
			break;
		case 39:
			engines.right = true;
			break;
	}

	updateState();
};

window.onkeyup = function(e) {
	if([ 37, 38, 39 ].indexOf(e.keyCode) === -1)
		return;

	e.preventDefault();

	switch(e.keyCode) {
		case 37:
			engines.left = false;
			break;
		case 38:
			engines.up = false;
			break;
		case 39:
			engines.right = false;
			break;
	}

	updateState();
};

// TOOD: ovládání myší

var mouseActive = false;
canvas.onmousedown = function(e) {
	mouseActive = true;

	e.preventDefault();
	handleMouseEvt(e);
};

function handleMouseEvt(e) {
	var x = e.offsetX / xRenderScale,
		y = (200 - e.offsetY / yRenderScale);

	followTarget = { x: x, y: y };
	mouseFollow();
}

canvas.onmousemove = function(e) {
	if(!mouseActive)
		return;

	e.preventDefault();
	handleMouseEvt(e);
};

window.onmouseup = function(e) {
	if(mouseActive)
		engines = { up: false, left: false, right: false };

	mouseActive = false;
};

var followTarget = null;

function mouseFollow() {
	if(!running || !mouseActive || !lastResp || !followTarget)
		return;

	var state = getState(lastResp, engines, (Date.now() - lastResp.timestamp) / 1000);

	if(followTarget.x < state.x - 10)
		engines.left = true;
	else
		engines.left = false;

	if(followTarget.x > state.x + 10)
		engines.right = true;
	else
		engines.right = false;

	if(followTarget.y > state.y + 10)
		engines.up = true;
	else
		engines.up = false;

	updateState();

}
setInterval(mouseFollow, 100);
